import os
import sys
from . import CheckBase

class Check(CheckBase):
    """
    Checks that there is no BEFORE_COMMIT or BEFORE-COMMIT file in the root
    of the working tree.
    """
    hooks = ["pre-commit"]

    def execute(self, hook):
        if os.environ.get("IGNORE_BEFORE_COMMIT"):
            return True

        ret = True
        if os.path.exists("BEFORE-COMMIT"):
            sys.stderr.write("BEFORE-COMMIT exists.\n")
            ret = False

        if os.path.exists("BEFORE_COMMIT"):
            sys.stderr.write("BEFORE_COMMIT exists.\n")
            ret = False

        return ret



import glerbl
import subprocess
from . import CheckBase

class Check(CheckBase):
    """
    Checks that none of the staged files contain trailing whitespaces.
    """
    hooks = ["pre-commit"]

    def execute(self, hook):
        against = glerbl.get_against()
        st = subprocess.call(["git", "diff-index", "--check", "--cached",
                              against, "--"],
                             stderr=subprocess.STDOUT)
        if st != 0:
            return False
        return True




class CheckBase(object):
    """
    Base class for checks.

    """

    hooks = []
    # pylint: disable=W0105
    """Git hooks to which this class applies. A list of strings."""

    def execute(self, hook):
        """
        Executes the check.

        :param hook: The name of the hook being run.
        :type hook: :class:`str`
        :returns: ``True`` if the check passed, ``False`` if not.
        :rtype: :class:`bool`

        """
        pass



import sys
import argparse
import importlib
import pkgutil
import stat
import os
from six.moves import input

GLERBL_DIR=".glerbl"
prog = None

class Exit(Exception):
    def __init__(self, status):
        super(Exit, self).__init__(status)

class NonExistentConfig(Exception):
    def __init__(self, path):
        super(NonExistentConfig, self).__init__(path)

def fatal(msg):
    sys.stderr.write("{0}: {1}\n".format(prog, msg))
    raise Exit(1)

def error(msg):
    sys.stderr.write("{0}: {1}\n".format(prog, msg))

def __load_config_file(path):
    if not os.path.exists(path):
        raise NonExistentConfig(path)

    glob = {"__file__": path}
    exec(compile(open(path).read(), path, 'exec'), glob)
    return glob

def __load_config():
    local_conf = os.path.join(GLERBL_DIR, "local_conf.py")
    repo_conf = os.path.join(GLERBL_DIR, "repo_conf.py")
    try:
        config = __load_config_file(local_conf)
        if not os.path.exists(repo_conf):
            fatal("{0} exists but not {1}.".format(local_conf, repo_conf))
    except NonExistentConfig:
        try:
            config = __load_config_file(repo_conf)
        except NonExistentConfig:
            fatal("cannot load {0} or {1}.".format(local_conf, repo_conf))
    return config

def __load_checks(check_names):
    checks = []
    for check_name in check_names:
        mod_name = "glerbl.check." + check_name \
                      if check_name.find(".") == -1 else check_name

        try:
            checks.append(importlib.import_module(mod_name))
        except ImportError:
            fatal("cannot import the module for check name {0}."
                  .format(check_name))
    return checks

def __verify_checks_for_hook(checks, hook):
    err = False
    for check in checks:
        if hook not in check.Check.hooks:
            err = True
            error("{0} not among those hooks supported by {1}."
                  .format(hook, check.__name__))
    if err:
        fatal("invalid configuration.")

def _runhook(which):
    config = __load_config()
    checks = __load_checks(config["checks"].get(which, []))
    __verify_checks_for_hook(checks, which)

    failed = False
    for check in checks:
        check_obj = check.Check()
        if not check_obj.execute(which):
            error(check.__name__ + " failed.")
            failed = True

    if failed:
        raise Exit(1)
    raise Exit(0)

def _install(_):
    """
    Install the hooks configured in glerbl.conf.py.

    """
    if not os.path.exists(".git"):
        fatal("not in a git repo.")

    config = __load_config()

    if config.get("checks") is None:
        fatal("global ``checks`` variable missing from configuration.")

    # We try to load all checks to check that they exist NOW.
    for hook in config["checks"]:
        checks = __load_checks(config["checks"][hook])
        # Also verify that they apply to the hooks.
        __verify_checks_for_hook(checks, hook)

    reply = input("This will overwrite your current hooks. "
                  "Proceed [yes/No]? ")
    if reply != "yes":
        raise Exit(0)

    for hook in config["checks"]:
        hook_f = open(os.path.join(".git/hooks/", hook), 'w')
        hook_f.write("""#!/usr/bin/env python
# GENRATED BY GLERBL. DO NOT MODIFY.
from glerbl.__main__ import main


exit(main(["glerbl", "hook", "{0}"]))
""".format(hook))
        os.fchmod(hook_f.fileno(),
                  os.fstat(hook_f.fileno()).st_mode | stat.S_IXUSR)
        hook_f.close()

def _listchecks(_):
    """
    Outputs to stdout the list of checks that glerbl knows about. The
    format of the list is::

        module_name [list of hooks]

           docstring

    The identifier ``module_name`` is the name of the module
    implementing the check. This name is how you'd refer to the check
    in your ``glerbl.conf.py`` file. The ``[list of hooks]`` is a list
    of hook name to which this check can be applied. The ``docstring``
    is a description of what the hook does.

    """
    import glerbl.check
    package = glerbl.check
    for _, modname, _ in pkgutil.walk_packages(path=package.__path__):
        mod = importlib.import_module(package.__name__+ '.' + modname)
        print(modname, mod.Check.hooks)
        print(mod.Check.__doc__)

def main(args=None):
    try:
        if args is None:
            args = sys.argv
        global prog # pylint: disable=W0603
        prog = sys.argv[0]
        if args and args[1] == "hook":
            if len(args) != 3:
                fatal("hook command requires 2 arguments.")
            _runhook(args[2])

        parser = argparse.ArgumentParser('Glerbl')
        subparsers = parser.add_subparsers(title="subcommands",
                                           help="subcommand")

        install_sp = subparsers.add_parser(
            "install",
            description=_install.__doc__,
            help="installs glerbl",
            formatter_class=argparse.RawTextHelpFormatter)
        install_sp.set_defaults(func=_install)

        listcheck_sp = subparsers.add_parser(
            "listchecks",
            description=_listchecks.__doc__,
            help="lists the checks that glerbl knows",
            formatter_class=argparse.RawTextHelpFormatter)
        listcheck_sp.set_defaults(func=_listchecks)

        args = parser.parse_args()

        args.func(args)
    except Exit as e:
        return e.args[0]
    return 0

if __name__ == '__main__':
    exit(main())



import glerbl
import subprocess
import os
import sys
from . import CheckBase

class Check(CheckBase):
    """
    Checks that no Python file among those staged contain a trailing
    semicolon. Pylint is used for this check. So if you run a more
    comprehensive pylint check, you might want to skip this check and
    have your other check include the semicolon check.
    """
    hooks = ["pre-commit"]

    def execute(self, hook):
        against = glerbl.get_against()
        tmpdir = glerbl.tree_from_staged()

        fail = 0

        for line in subprocess.check_output(["git", "diff-index", "--cached",
                                             "--full-index", against],
                                            universal_newlines=True).split("\n"):
            line = line.strip()
            if line == "":
                continue

            (_, _, _, _, status, filename) = line.split()

            ext = os.path.splitext(filename)[1]

            if ext != ".py" or status == "D":
                continue

            stdout = open(os.devnull, 'w')
            lint_st = subprocess.call(["pylint", "--disable=E,C,W,R",
                                       "--enable=W0301",
                                       os.path.join(tmpdir, filename)],
                                      stdout=stdout)
            if lint_st != 0:
                sys.stderr.write(filename +
                                 " contains trailing semicolons; or was so "
                                 "mangled that pylint failed.\n")
                fail = True

        if fail:
            return False

        return True



import glerbl
import subprocess
import sys
import re
import six
from . import CheckBase

allownonascii = False
try:
    allownonascii = (subprocess.check_output(["git", "config",
                                              "hooks.allownonascii"])
                     .strip() == b"true")
except subprocess.CalledProcessError:
    pass

non_ascii_re = re.compile(r"[^ -~]")

class Check(CheckBase):
    """
    Checks that no file with non-ASCII filenames are staged.
    """
    hooks = ["pre-commit"]

    def execute(self, hook):
        if allownonascii:
            return True

        against = glerbl.get_against()

        output = subprocess.check_output(
            ["git", "diff", "--cached", "--name-only",
             "--diff-filter=A", "-z", against])

        names = (output if not six.PY3 else output.decode("utf-8")).split("\x00")
        good = True
        for name in [x for x in names if non_ascii_re.search(x) is not None]:
            sys.stderr.write("{0} is a non-ascii filename.\n".format(name))
            good = False

        return good



from setuptools import setup, find_packages

long_description = ""
found = False
with open("README.rst", 'r') as readme:
    captured = []
    for line in readme:
        if line == \
           ".. Everything above goes into setup.py's long_description.\n":
            found = True
            break
        captured.append(line)

    long_description = "".join(captured)

if not found:
    raise ValueError("can't find end of long description in README.rst")

setup(
    name="glerbl",
    version="0.1.0",
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'glerbl = glerbl.__main__:main'
        ],
    },
    author="Louis-Dominique Dubeau",
    author_email="ldd@lddubeau.com",
    description="Glerbl manages git hooks.",
    license="GPLv3+",
    keywords=["git", "git hooks"],
    url="https://github.com/lddubeau/glerbl",
    install_requires=[
        'nose>=1.3.0',
        'fresher>=0.3.1',
        'six'
    ],
    #use_2to3=True,
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: POSIX",
        "Topic :: Software Development :: Version Control",
        "Topic :: Software Development :: Quality Assurance"
    ],
    long_description=long_description
)



import os
import shutil
import subprocess
import atexit
import tempfile

# No warning about globals in this file.
# pylint: disable=W0603

__tmpdir = None

def __clean_tmpdir():
    global __tmpdir
    shutil.rmtree(__tmpdir, ignore_errors=True)
    __tmpdir = None

def get_tmpdir():
    """
    On first invocation, creates a temporary directory and returns its
    path. Subsequent invocations uses the same directory.

    :returns: A temporary directory created for this run of glerbl.
    :rtype: :class:`str`
    """
    global __tmpdir
    if __tmpdir is not None:
        return __tmpdir

    __tmpdir = tempfile.mkdtemp(prefix='.tmp.glerbl.', dir=".")
    atexit.register(__clean_tmpdir)

    return __tmpdir

__cached_against = None

def get_against():
    """
    Determines the revision against which the staged data ought to be checked.

    :returns: The revision.
    :rtype: :class:`str`
    """
    global __cached_against
    if __cached_against is not None:
        return __cached_against

    status = subprocess.call(["git", "rev-parse", "--verify", "HEAD"],
                             stdout=open(os.devnull, 'w'),
                             stderr=subprocess.STDOUT)
    if not status:
        against = 'HEAD'
    else:
	# Initial commit: diff against an empty tree object
        against = '4b825dc642cb6eb9a060e54bf8d69288fbee4904'

    __cached_against = against
    return against


__cached_tree_from_staged = None


def tree_from_staged():
    global __cached_tree_from_staged
    if __cached_tree_from_staged is not None:
        return __cached_tree_from_staged

    against = get_against()

    tmpdir = os.path.join(get_tmpdir(), "tree_from_staged")

    if os.path.exists(tmpdir):
        raise Exception(tmpdir + " already exists")

    os.mkdir(tmpdir)

    # Link all the files that git already knows about.
    for path in subprocess.check_output(["git", "ls-files"],
                                        universal_newlines=True).split("\n"):
        path = path.strip()
        if path == "":
            continue

        dest = os.path.join(tmpdir, path)
        dirname = os.path.dirname(dest)
        if dirname != "" and not os.path.exists(dirname):
            os.makedirs(dirname)

        # Don't try to link to files that don't exist anymore.
        if os.path.exists(path):
            os.link(path, dest)

    # Modify the tree according to what is staged.
    for line in subprocess.check_output(["git", "diff-index", "--cached",
                                         "--full-index", against],
                                        universal_newlines=True).split("\n"):
        line = line.strip()
        if line == "":
            continue

        (_, _, _, sha, status, filename) = line.split()

        dest = os.path.join(tmpdir, filename)
        if os.path.exists(dest):
            if os.path.isdir(dest):
                shutil.rmtree(dest)
            else:
                os.unlink(dest)

        # Deleted file, we're done.
        if status == "D":
            continue

        subprocess.check_call(["git", "cat-file", "-p", sha],
                              stdout=open(dest, 'w'))

    __cached_tree_from_staged = tmpdir
    return tmpdir



import subprocess
import os
import sys

import glerbl
from . import CheckBase

class Check(CheckBase):
    """
    Checks that non of the staged files have PEP8 errors. It runs the
    ``pep8`` tool on all the stages files.
    """

    hooks = ["pre-commit"]

    def execute(self, hook):
        against = glerbl.get_against()

        fail = False

        for line in subprocess.check_output(["git", "diff-index", "--cached",
                                             "--full-index", against],
                                            universal_newlines=True) \
                              .split("\n"):
            line = line.strip()
            if line == "":
                continue

            (_, _, _, sha, status, filename) = line.split()

            ext = os.path.splitext(filename)[1]

            if ext != ".py" or status == "D":
                continue

            pep_ret = os.system(("git cat-file -p {0} | "
                                "pep8 /dev/stdin > /dev/null").format(sha))
            if pep_ret != 0:
                sys.stderr.write(filename + " does not comply with pep8.\n")
                fail = True

        if fail:
            return False
        return True
